/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;


/**
 * This interface defines the contract for classes wishing to discover the current node.  Implementations of this
 * interface will be called when the current page has not been defined (in a page scope attribute for {@link
 * BreadCrumbTrailTag#BREADCRUMB_CURRENT_NODE}). The implementation should discover, in it's own way, the current node
 * when {@link #getCurrentNode(PageContext)} is called and return the node representation of that url.
 * <p/>
 * <b>Thread Safety:</b> Implementations should be thread safe in the context of the JSP rendering process
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public interface NodeDiscovery {
    /**
     * This method is called to retrieve the node representation of the current page. If the current page could not be
     * found or is not applicable - simply return <tt>null</tt>. The returned node should include any query strings
     * available.
     *
     * @param pageContext a non-<tt>null</tt> page context to use
     *
     * @return a {@link BreadCrumbNode} representing the current url or <tt>null</tt> if the node couldn't be
     *         determined
     *
     * @throws IllegalArgumentException if the page context is <tt>null</tt>
     * @throws BreadCrumbException      if an exception occurs discovering the page
     */
    public BreadCrumbNode getCurrentNode(PageContext pageContext) throws BreadCrumbException;
}


