/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;


/**
 * This interface defines the contract for classes wishing to discover the path from current node to the root node.
 * Implementations of this interface will be called with the current page (specified by the current node) and the
 * current {@link PageContext}.  The implementation is then supposed to discover the complete path from the current node
 * to the root or return <tt>null</tt> when it can't discover the current path.
 * <p/>
 * <b>Thread Safety:</b> Implementations should be thread safe in the context of the JSP rendering process.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public interface PathDiscovery {
    /**
     * This method should return the path from the current node for the given page context.  The current node can have a
     * <tt>null</tt> title if the current node should default to that found in the data source for this implementation.
     * The returned nodes, however, should have a title assigned to each node.  The current node's url can include query
     * strings on both the current node that is called and in the nodes that are returned in the path.  Implementors of
     * this method should return an array where the 0th index position is the 'root' node and the nth-1 index position
     * is the current node or return <tt>null</tt> if the current path could not be found.
     *
     * @param currentNode A non-<tt>null</tt> node representing the current page
     * @param pageContext The non-<tt>null</tt> page context
     *
     * @return An array of {@link BreadCrumbNode}s representing the path from root (0th index) to current node (nth-1
     *         index) or <tt>null</tt> if path couldn't be found
     *
     * @throws IllegalArgumentException if either arg is <tt>null</tt>
     * @throws BreadCrumbException      if an exception occurs discovering the page
     */
    public BreadCrumbNode[] getPath(BreadCrumbNode currentNode, PageContext pageContext) throws BreadCrumbException;
}


