/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.AbstractNodeDiscovery;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.PageContext;


/**
 * This tag and implementation of NodeDiscovery will discover the current node from the {@link HttpServletRequest}. This
 * implementation will return a valid URI (with query string if specified) in the {@link #getCurrentNode(PageContext)}
 * method.
 * <p/>
 * If this is used as a tag, this will define a page scope variable called  {@link
 * com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag#BREADCRUMB_CURRENT_NODE} with a string representing the
 * node URI with query string - see {@link AbstractNodeDiscovery} for details on this.
 * <p/>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class HttpServletNodeDiscoveryTag extends AbstractNodeDiscovery {

    /**
     * Constructs the HttpServletNodeDiscoveryTag using no constructor parameters.
     */
    public HttpServletNodeDiscoveryTag() {
        super();
    }

    /**
     * This method discovers the node url from an {@link HttpServletRequest}. If none is found or if the given {@link
     * PageContext}'s request is not of type {@link HttpServletRequest} type, <tt>null</tt> is returned.
     *
     * @param pageContext A non-<tt>null</tt> page context to use
     *
     * @return <tt>null</tt> if not applicable or a non-<tt>null</tt>, non-empty string representing the contextual URI
     *         (with query string)
     *
     * @throws IllegalArgumentException if the pageContext is <tt>null</tt>
     */
    public BreadCrumbNode getCurrentNode(final PageContext pageContext) {
        if (pageContext == null) {
            throw new IllegalArgumentException("The parameter named [pageContext] was null.");
        }

        final ServletRequest request = pageContext.getRequest();
        if (request instanceof HttpServletRequest) {
            final HttpServletRequest httpServletRequest = (HttpServletRequest) request;
            final String queryString = httpServletRequest.getQueryString();
            return new BreadCrumbNode(
                httpServletRequest.getRequestURI() + (queryString != null ? "?" + queryString : ""), getTitle());
        }
        return null;
    }
}
