/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import javax.servlet.jsp.tagext.TagSupport;
import java.io.IOException;


/**
 * This tag will format the node and title using the html anchor pattern of <tt>&lt;a
 * href=&quot;node.getURL()&quot;&gt;title&lt;/a&gt;</tt>.
 * <p/>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class HtmlAnchorNodeFormatterTag extends TagSupport {

    /**
     * Constructs the HtmlAnchorNodeFormatterTag using no constructor parameters.
     */
    public HtmlAnchorNodeFormatterTag() {
        super();
    }

    /**
     * This method will format the current node and style as a simple html anchor and then write the results out.
     *
     * @return Always {@link #SKIP_BODY}
     *
     * @throws BreadCrumbException in case the page context does not contain the information necessary to render the
     *                             anchor
     */
    public int doStartTag() throws BreadCrumbException {
        //retrieve the node to be rendered
        final BreadCrumbNode breadCrumbNode = (BreadCrumbNode) NodeFormatterUtils.getRequiredPageContextAttribute(
            pageContext, BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, BreadCrumbNode.class);

        // retrieve the style class name to be used
        String styleStr = (String) NodeFormatterUtils.getOptionalPageContextAttribute(
            pageContext, BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, String.class);
        if (styleStr != null && styleStr.trim().length() == 0) {
            styleStr = null;
        }

        // create the anchor tag
        final StringBuffer ret = new StringBuffer();
        ret.append("<a ");
        if (styleStr != null) {
            ret.append("class=\"");
            ret.append(styleStr);
            ret.append("\" ");
        }
        ret.append("href=\"");
        ret.append(breadCrumbNode.getUrl());
        ret.append("\">");
        final String title = breadCrumbNode.getTitle();
        if (title != null) {
            ret.append(title);
        } else {
            ret.append("Unspecified");
        }
        ret.append("</a>");

        //write the anchor tag out
        try {
            pageContext.getOut().write(ret.toString());
        } catch (IOException e) {
            throw new BreadCrumbException("Error while writing the rendered anchor tag.", e);
        }

        return SKIP_BODY;
    }

}
