/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import junit.framework.TestCase;


/**
 * This class contains all test cases for {@link com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class BreadCrumbNodeTest extends TestCase {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";
    /**
     * This is an empty string for checking empty String argument behavior.
     */
    private static final String EMPTY_STRING = "  ";

    /**
     * This method tests the correct behavior of {@link BreadCrumbNode#BreadCrumbNode(String,String)}.
     */
    public void testCtor() {
        BreadCrumbNode node = new BreadCrumbNode(URL, TITLE);
        assertEquals(TITLE, node.getTitle());
        assertEquals(URL, node.getUrl());

        // check with empty title
        node = new BreadCrumbNode(URL, EMPTY_STRING);
        assertEquals(null, node.getTitle());
        assertEquals(URL, node.getUrl());
        // check with null title
        node = new BreadCrumbNode(URL, null);
        assertEquals(null, node.getTitle());
        assertEquals(URL, node.getUrl());
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbNode#BreadCrumbNode(String,String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> url argument
     */
    public void testCtorFailureNullUrl() {
        try {
            new BreadCrumbNode(null, TITLE);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbNode#BreadCrumbNode(String,String)}.
     * <p/>
     * <b>Failure reason</b>: empty url argument
     */
    public void testCtorFailureEmptyUrl() {
        try {
            new BreadCrumbNode(EMPTY_STRING, TITLE);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbNode#getUrl()}.
     */
    public void testGetUrl() {
        final BreadCrumbNode node = new BreadCrumbNode(URL, TITLE);
        assertEquals(URL, node.getUrl());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbNode#getTitle()}.
     */
    public void testGetTitle() {
        BreadCrumbNode node = new BreadCrumbNode(URL, TITLE);
        assertEquals(TITLE, node.getTitle());
        // check with empty title
        node = new BreadCrumbNode(URL, EMPTY_STRING);
        assertEquals(null, node.getTitle());
        // check with null title
        node = new BreadCrumbNode(URL, null);
        assertEquals(null, node.getTitle());
    }
}
