/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import com.meterware.httpunit.WebResponse;
import com.topcoder.util.config.ConfigManager;
import org.apache.cactus.server.PageContextWrapper;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.el.ExpressionEvaluator;
import javax.servlet.jsp.el.VariableResolver;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;
import java.io.IOException;


/**
 * This class contains all test cases for {@link BreadCrumbTrailTag}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class BreadCrumbTrailTagTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is a value used as valid argument in string attribute assignment testcases.
     */
    private static final String VALUE = "__Value--";
    /**
     * This is an empty string for checking empty String argument behavior.
     */
    private static final String EMPTY_STRING = "  ";
    /**
     * This is the url of node 1 from the site map used in test case.
     */
    private static final String NODE_1_URL = "uu1";
    /**
     * This is the title of node 1 from the site map used in test case.
     */
    private static final String NODE_1_TITLE = "aa1";
    /**
     * This is the url of node 2 from the site map used in test case.
     */
    private static final String NODE_2_URL = "uu2";
    /**
     * This is the title of node 2 from the site map used in test case.
     */
    private static final String NODE_2_TITLE = "aa2";
    /**
     * This is the url of node 5 from the site map used in test case.
     */
    private static final String NODE_5_URL = "uu5";
    /**
     * This is the title of node 5 from the site map used in test case.
     */
    private static final String NODE_5_TITLE = "aa5";
    /**
     * This is the url of node 6 from the site map used in test case.
     */
    private static final String NODE_6_URL = "uu6";
    /**
     * This is the title of node 6 from the site map used in test case.
     */
    private static final String NODE_6_TITLE = "a";
    /**
     * This is the url of node 7 from the site map used in test case.
     */
    private static final String NODE_7_URL = "uu7";
    /**
     * This is the title of node 7 from the site map used in test case.
     */
    private static final String NODE_7_TITLE = "aa7";
    /**
     * This is the instance to be tested, initialized in {@link #setUp()}.
     */
    private BreadCrumbTrailTag breadCrumbTrailTag;

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getPathSeparator()} and {@link
     * BreadCrumbTrailTag#setPathSeparator(String)} .
     */
    public void testGetSetPathSeparator() {
        assertEquals(":", breadCrumbTrailTag.getPathSeparator());
        breadCrumbTrailTag.setPathSeparator(VALUE);
        assertEquals(VALUE, breadCrumbTrailTag.getPathSeparator());
        breadCrumbTrailTag.setPathSeparator(EMPTY_STRING);
        assertEquals(EMPTY_STRING, breadCrumbTrailTag.getPathSeparator());
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbTrailTag#setPathSeparator(String)} .
     * <p/>
     * <b>Failure scenario:</b>pathSeparator arg is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testSetPathSeparatorFailureNull() throws Exception {
        try {
            breadCrumbTrailTag.setPathSeparator(null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getPathDirection()} and {@link
     * BreadCrumbTrailTag#setPathDirection(int)} .
     */
    public void testGetSetPathDirection() {
        assertEquals(BreadCrumbTrailTag.ROOT_TO_CURRENT, breadCrumbTrailTag.getPathDirection());
        breadCrumbTrailTag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);
        assertEquals(BreadCrumbTrailTag.CURRENT_TO_ROOT, breadCrumbTrailTag.getPathDirection());
        breadCrumbTrailTag.setPathDirection(BreadCrumbTrailTag.ROOT_TO_CURRENT);
        assertEquals(BreadCrumbTrailTag.ROOT_TO_CURRENT, breadCrumbTrailTag.getPathDirection());
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbTrailTag#setPathDirection(int)} .
     * <p/>
     * <b>Failure scenario:</b>pathDirection arg is invalid
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testSetPathDirectionFailureNull() throws Exception {
        try {
            breadCrumbTrailTag.setPathDirection(5);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getPathSeparatorStyle()} and {@link
     * BreadCrumbTrailTag#setPathSeparatorStyle(String)} .
     */
    public void testGetSetPathSeparatorStyle() {
        assertEquals(null, breadCrumbTrailTag.getPathSeparatorStyle());
        breadCrumbTrailTag.setPathSeparatorStyle(VALUE);
        assertEquals(VALUE, breadCrumbTrailTag.getPathSeparatorStyle());
        breadCrumbTrailTag.setPathSeparatorStyle(null);
        assertEquals(null, breadCrumbTrailTag.getPathSeparatorStyle());
        breadCrumbTrailTag.setPathSeparatorStyle(EMPTY_STRING);
        assertEquals(null, breadCrumbTrailTag.getPathSeparatorStyle());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getCurrentNodeStyle()} and {@link
     * BreadCrumbTrailTag#setCurrentNodeStyle(String)} .
     */
    public void testGetSetCurrentNodeStyle() {
        assertEquals(null, breadCrumbTrailTag.getCurrentNodeStyle());
        breadCrumbTrailTag.setCurrentNodeStyle(VALUE);
        assertEquals(VALUE, breadCrumbTrailTag.getCurrentNodeStyle());
        breadCrumbTrailTag.setCurrentNodeStyle(null);
        assertEquals(null, breadCrumbTrailTag.getCurrentNodeStyle());
        breadCrumbTrailTag.setCurrentNodeStyle(EMPTY_STRING);
        assertEquals(null, breadCrumbTrailTag.getCurrentNodeStyle());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getNodeStyle()} and {@link
     * BreadCrumbTrailTag#setNodeStyle(String)} .
     */
    public void testGetSetNodeStyle() {
        assertEquals(null, breadCrumbTrailTag.getNodeStyle());
        breadCrumbTrailTag.setNodeStyle(VALUE);
        assertEquals(VALUE, breadCrumbTrailTag.getNodeStyle());
        breadCrumbTrailTag.setNodeStyle(null);
        assertEquals(null, breadCrumbTrailTag.getNodeStyle());
        breadCrumbTrailTag.setNodeStyle(EMPTY_STRING);
        assertEquals(null, breadCrumbTrailTag.getNodeStyle());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getRootNodeStyle()} and {@link
     * BreadCrumbTrailTag#setRootNodeStyle(String)} .
     */
    public void testGetSetRootNodeStyle() {
        assertEquals(null, breadCrumbTrailTag.getRootNodeStyle());
        breadCrumbTrailTag.setRootNodeStyle(VALUE);
        assertEquals(VALUE, breadCrumbTrailTag.getRootNodeStyle());
        breadCrumbTrailTag.setRootNodeStyle(null);
        assertEquals(null, breadCrumbTrailTag.getRootNodeStyle());
        breadCrumbTrailTag.setRootNodeStyle(EMPTY_STRING);
        assertEquals(null, breadCrumbTrailTag.getRootNodeStyle());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getMouseOverStyle()} and {@link
     * BreadCrumbTrailTag#setMouseOverStyle(String)} .
     */
    public void testGetSetMouseOverStyle() {
        assertEquals(null, breadCrumbTrailTag.getMouseOverStyle());
        breadCrumbTrailTag.setMouseOverStyle(VALUE);
        assertEquals(VALUE, breadCrumbTrailTag.getMouseOverStyle());
        breadCrumbTrailTag.setMouseOverStyle(null);
        assertEquals(null, breadCrumbTrailTag.getMouseOverStyle());
        breadCrumbTrailTag.setMouseOverStyle(EMPTY_STRING);
        assertEquals(null, breadCrumbTrailTag.getMouseOverStyle());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getPathOverride()} and {@link
     * BreadCrumbTrailTag#setPathOverride(String)} .
     */
    public void testGetSetPathOverride() {
        assertEquals(null, breadCrumbTrailTag.getPathOverride());
        breadCrumbTrailTag.setPathOverride(VALUE);
        assertEquals(VALUE, breadCrumbTrailTag.getPathOverride());
        breadCrumbTrailTag.setPathOverride(null);
        assertEquals(null, breadCrumbTrailTag.getPathOverride());
        breadCrumbTrailTag.setPathOverride(EMPTY_STRING);
        assertEquals(null, breadCrumbTrailTag.getPathOverride());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getSilentErrors()} and {@link
     * BreadCrumbTrailTag#setSilentErrors(boolean)} .
     */
    public void testGetSetSilentErrors() {
        assertEquals(false, breadCrumbTrailTag.getSilentErrors());
        breadCrumbTrailTag.setSilentErrors(true);
        assertEquals(true, breadCrumbTrailTag.getSilentErrors());
        breadCrumbTrailTag.setSilentErrors(false);
        assertEquals(false, breadCrumbTrailTag.getSilentErrors());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#getTitle()} and {@link
     * BreadCrumbTrailTag#setTitle(String)} .
     */
    public void testGetSetTitle() {
        assertEquals(null, breadCrumbTrailTag.getTitle());
        breadCrumbTrailTag.setTitle(VALUE);
        assertEquals(VALUE, breadCrumbTrailTag.getTitle());
        breadCrumbTrailTag.setTitle(null);
        assertEquals(null, breadCrumbTrailTag.getTitle());
        breadCrumbTrailTag.setTitle(EMPTY_STRING);
        assertEquals(null, breadCrumbTrailTag.getTitle());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#BreadCrumbTrailTag()}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtorConfigLoading() throws Exception {
        loadTagConfiguration("breadcrumb_tagConfiguration.xml");
        breadCrumbTrailTag = new BreadCrumbTrailTag();

        assertEquals("__pathSeparator__", breadCrumbTrailTag.getPathSeparator());
        assertEquals("__pathSeparatorStyle__", breadCrumbTrailTag.getPathSeparatorStyle());
        assertEquals("__nodeStyle__", breadCrumbTrailTag.getNodeStyle());
        assertEquals("__currentNodeStyle__", breadCrumbTrailTag.getCurrentNodeStyle());
        assertEquals("__rootNodeStyle__", breadCrumbTrailTag.getRootNodeStyle());
        assertEquals("__mouseOverStyle__", breadCrumbTrailTag.getMouseOverStyle());
        assertEquals(true, breadCrumbTrailTag.getSilentErrors());
        assertEquals(1, breadCrumbTrailTag.getPathDirection());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doStartTag()}.
     * <p/>
     * <b>Special Scenario</b>: Node cannot be discovered
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagNodeDiscoveryFailed() throws Exception {
        breadCrumbTrailTag.setPageContext(new DummyPageContextWrapper());
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doStartTag());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doStartTag()}.
     * <p/>
     * <b>Special Scenario</b>: Path cannot be discovered
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagPathDiscoveryFailed() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode("blah", null));
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doStartTag());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doStartTag()}.
     * <p/>
     * <b>Special Scenario</b>: default configuration and default behavior
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTag() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_1_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTag()} has been executed to verify the
     * output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link BreadCrumbTrailTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>default configuration and default behavior
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTag(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doStartTag()}.
     * <p/>
     * <b>Special Scenario</b>: default configuration and path override
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagPathOverride() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_1_URL, null));
        breadCrumbTrailTag.setPathOverride(
            "<BreadCrumb xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
                + "xsi:noNamespaceSchemaLocation=\"BreadCrumbTrailSchema.xsd\" version=\"v2\">\n"
                + " <bc_node title=\"aa1\" url=\"uu1\" pattern=\"uu1\"/>"
                + "</BreadCrumb>");
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagPathOverride()} has been executed to
     * verify the output generated by {@link #testDoStartTagPathOverride()}. Thus it indirectly tests {@link
     * BreadCrumbTrailTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>default configuration and path override
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagPathOverride(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doStartTag()}.
     * <p/>
     * <b>Special Scenario</b>: styles have been configured
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagWithStyles() throws Exception {
        loadTagConfiguration("breadcrumb_tagConfiguration.xml");
        breadCrumbTrailTag = new BreadCrumbTrailTag();
        breadCrumbTrailTag.setPathOverride(null);
        breadCrumbTrailTag.setPageContext(pageContext);

        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_1_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagWithStyles()} has been executed to
     * verify the output generated by {@link #testDoStartTagWithStyles()}. Thus it indirectly tests {@link
     * BreadCrumbTrailTag#doStartTag()}
     * <p/>
     * <b>Special Scenario</b>: styles have been configured
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagWithStyles(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("<span><style>.path-separator-style {__pathSeparatorStyle__}\n"
            + ".node-style {__nodeStyle__}\n"
            + ".current-node-style {__currentNodeStyle__}\n"
            + ".root-node-style {__rootNodeStyle__}\n"
            + ".node-style:hover {__mouseOverStyle__}\n"
            + ".root-node-style:hover {__mouseOverStyle__}\n"
            + ".current-node-style:hover {__mouseOverStyle__}\n"
            + "</style>\n", resp.getText());
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbTrailTag#doStartTag()} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory configuration is missing
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagFailNoObjectFactoryConfig() throws Exception {
        clearConfig();
        breadCrumbTrailTag.setSilentErrors(true);
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.setSilentErrors(false);
        try {
            breadCrumbTrailTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbTrailTag#doStartTag()} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory configuration is invalid
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagFailInvalidObjectFactoryConfig() throws Exception {
        loadTagConfiguration("breadcrumb_invalidObjectFactoryConfig.xml");
        breadCrumbTrailTag.setSilentErrors(true);
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.setSilentErrors(false);
        try {
            breadCrumbTrailTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbTrailTag#doStartTag()} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory configuration contains unconstructable NodeDiscovery specification
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagFailUnconstructableNodeDiscoveryObjectFactoryConfig() throws Exception {
        loadTagConfiguration("breadcrumb_unconstructableNodeDiscovery.xml");
        breadCrumbTrailTag.setSilentErrors(true);
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.setSilentErrors(false);
        try {
            breadCrumbTrailTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbTrailTag#doStartTag()} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory configuration contains unconstructable PathDiscovery specification
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagFailUnconstructablePathDiscoveryObjectFactoryConfig() throws Exception {
        loadTagConfiguration("breadcrumb_unconstructablePathDiscovery.xml");
        breadCrumbTrailTag.setSilentErrors(true);
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.setSilentErrors(false);
        try {
            breadCrumbTrailTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link BreadCrumbTrailTag#doStartTag()} .
     * <p/>
     * <b>Failure scenario:</b>specified pathOverride is unparseable
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagFailUnparseablePathOverride() throws Exception {
        breadCrumbTrailTag.setPathOverride("blah");
        breadCrumbTrailTag.setSilentErrors(true);
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.setSilentErrors(false);
        try {
            breadCrumbTrailTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doInitBody()}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoInitBody() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_1_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
        //default
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        breadCrumbTrailTag.doInitBody();
        assertEqualNode(new BreadCrumbNode(NODE_1_URL, NODE_1_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));

        // root node style set
        breadCrumbTrailTag.setRootNodeStyle("blah");
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.doInitBody();
        assertEquals("root-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        breadCrumbTrailTag.setRootNodeStyle(null);

        // mouse over style set
        breadCrumbTrailTag.setMouseOverStyle("blah");
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.doInitBody();
        assertEquals("root-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        breadCrumbTrailTag.setMouseOverStyle(null);
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doInitBody()}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoInitBodyReverse() throws Exception {

        //reverse direction
        breadCrumbTrailTag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_2_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.doInitBody();
        assertEqualNode(new BreadCrumbNode(NODE_2_URL, NODE_2_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));

        // current node style set
        breadCrumbTrailTag.setCurrentNodeStyle("blah");
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.doInitBody();
        assertEquals("current-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        breadCrumbTrailTag.setCurrentNodeStyle(null);

        // mouse over style set
        breadCrumbTrailTag.setMouseOverStyle("blah");
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());
        breadCrumbTrailTag.doInitBody();
        assertEquals("current-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        breadCrumbTrailTag.setMouseOverStyle(null);
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doAfterBody()}.
     * <p/>
     * <b>Special Scenario</b>: default behavior with no styles
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoAfterBody() throws Exception {
        final BodyContent bodyContent = pageContext.pushBody();
        breadCrumbTrailTag.setBodyContent(bodyContent);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_7_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());

        // initial node
        breadCrumbTrailTag.doInitBody();
        assertEqualNode(new BreadCrumbNode(NODE_5_URL, NODE_5_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node one\n");

        // first iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_6_URL, NODE_6_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node two\n");

        // second iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_7_URL, NODE_7_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node three\n");

        //iteration finished
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doAfterBody());

        pageContext.popBody();
    }

    /**
     * This method is called by the cactus framework after {@link #testDoAfterBody()} has been executed to verify the
     * output generated by {@link #testDoAfterBody()}. Thus it indirectly tests {@link
     * BreadCrumbTrailTag#doAfterBody()}
     * <p/>
     * <b>Special Scenario</b>: default behavior with no styles
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoAfterBody(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals(
            "node one\n"
                + ":node two\n"
                + ":node three\n", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doAfterBody()}.
     * <p/>
     * <b>Special Scenario</b>: default behavior with styles
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoAfterBodyWithStyles() throws Exception {
        final BodyContent bodyContent = pageContext.pushBody();

        breadCrumbTrailTag.setBodyContent(bodyContent);
        breadCrumbTrailTag.setNodeStyle("__nodeStyle__");
        breadCrumbTrailTag.setCurrentNodeStyle("__currentNodeStyle__");
        breadCrumbTrailTag.setRootNodeStyle("__RootNodeStyle__");
        breadCrumbTrailTag.setPathSeparatorStyle("__PathSeparatorStyle__");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_7_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());

        // initial node
        breadCrumbTrailTag.doInitBody();
        assertEqualNode(new BreadCrumbNode(NODE_5_URL, NODE_5_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("root-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node one\n");

        // first iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_6_URL, NODE_6_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node two\n");

        // second iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_7_URL, NODE_7_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("current-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node three\n");

        //iteration finished
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doAfterBody());

        pageContext.popBody();
    }

    /**
     * This method is called by the cactus framework after {@link #testDoAfterBodyWithStyles()} has been executed to
     * verify the output generated by {@link #testDoAfterBodyWithStyles()}. Thus it indirectly tests {@link
     * BreadCrumbTrailTag#doAfterBody()}
     * <p/>
     * <b>Special Scenario</b>: default behavior with styles
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoAfterBodyWithStyles(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals(
            "<span><style>.path-separator-style {__PathSeparatorStyle__}\n"
                + ".node-style {__nodeStyle__}\n"
                + ".current-node-style {__currentNodeStyle__}\n"
                + ".root-node-style {__RootNodeStyle__}\n"
                + "</style>\n"
                + "node one\n"
                + "<span class=\"path-separator-style\">:</span>node two\n"
                + "<span class=\"path-separator-style\">:</span>node three\n", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doAfterBody()}.
     * <p/>
     * <b>Special Scenario</b>: default behavior with only mouse over style set
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoAfterBodyWithMouseOverStyle() throws Exception {
        final BodyContent bodyContent = pageContext.pushBody();

        breadCrumbTrailTag.setBodyContent(bodyContent);
        breadCrumbTrailTag.setMouseOverStyle("__mouseOverStyle__");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_7_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());

        // initial node
        breadCrumbTrailTag.doInitBody();
        assertEqualNode(new BreadCrumbNode(NODE_5_URL, NODE_5_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("root-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node one\n");

        // first iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_6_URL, NODE_6_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node two\n");

        // second iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_7_URL, NODE_7_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("current-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node three\n");

        //iteration finished
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doAfterBody());

        pageContext.popBody();
    }

    /**
     * This method is called by the cactus framework after {@link #testDoAfterBodyWithMouseOverStyle()} has been
     * executed to verify the output generated by {@link #testDoAfterBodyWithMouseOverStyle()}. Thus it indirectly tests
     * {@link BreadCrumbTrailTag#doAfterBody()}
     * <p/>
     * <b>Special Scenario</b>: default behavior with only mouse over style set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoAfterBodyWithMouseOverStyle(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals(
            "<span><style>.node-style:hover {__mouseOverStyle__}\n"
                + ".root-node-style:hover {__mouseOverStyle__}\n"
                + ".current-node-style:hover {__mouseOverStyle__}\n"
                + "</style>\n"
                + "node one\n"
                + ":node two\n"
                + ":node three\n", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doAfterBody()}.
     * <p/>
     * <b>Special Scenario</b>: default behavior with custom separator
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoAfterBodyWithCustomSeparator() throws Exception {
        final BodyContent bodyContent = pageContext.pushBody();

        breadCrumbTrailTag.setBodyContent(bodyContent);
        breadCrumbTrailTag.setPathSeparator("");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_7_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());

        // initial node
        breadCrumbTrailTag.doInitBody();
        assertEqualNode(new BreadCrumbNode(NODE_5_URL, NODE_5_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node one\n");

        // first iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_6_URL, NODE_6_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node two\n");

        // second iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_7_URL, NODE_7_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals(null, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node three\n");

        //iteration finished
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doAfterBody());

        pageContext.popBody();
    }

    /**
     * This method is called by the cactus framework after {@link #testDoAfterBodyWithCustomSeparator()} has been
     * executed to verify the output generated by {@link #testDoAfterBodyWithCustomSeparator()}. Thus it indirectly
     * tests {@link BreadCrumbTrailTag#doAfterBody()}
     * <p/>
     * <b>Special Scenario</b>: default behavior with only mouse over style set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoAfterBodyWithCustomSeparator(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals(
            "node one\n"
                + "node two\n"
                + "node three\n", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doAfterBody()}.
     * <p/>
     * <b>Special Scenario</b>: current to root behavior with only mouse over style set
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoAfterBodyCurrentToRoot() throws Exception {
        final BodyContent bodyContent = pageContext.pushBody();

        breadCrumbTrailTag.setBodyContent(bodyContent);
        breadCrumbTrailTag.setMouseOverStyle("__mouseOverStyle__");
        breadCrumbTrailTag.setPathDirection(BreadCrumbTrailTag.CURRENT_TO_ROOT);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, new BreadCrumbNode(NODE_7_URL, null));
        assertEquals(BodyTagSupport.EVAL_BODY_BUFFERED, breadCrumbTrailTag.doStartTag());

        // initial node
        breadCrumbTrailTag.doInitBody();
        assertEqualNode(new BreadCrumbNode(NODE_7_URL, NODE_7_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("current-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node one\n");

        // first iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_6_URL, NODE_6_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node two\n");

        // second iteration
        assertEquals(BodyTagSupport.EVAL_BODY_AGAIN, breadCrumbTrailTag.doAfterBody());
        assertEqualNode(new BreadCrumbNode(NODE_5_URL, NODE_5_TITLE), pageContext.getAttribute(
            BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, PageContext.PAGE_SCOPE));
        assertEquals("root-node-style", pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS,
            PageContext.PAGE_SCOPE));
        bodyContent.print("node three\n");

        //iteration finished
        assertEquals(BodyTagSupport.SKIP_BODY, breadCrumbTrailTag.doAfterBody());

        pageContext.popBody();
    }

    /**
     * This method is called by the cactus framework after {@link #testDoAfterBodyCurrentToRoot()} has been executed to
     * verify the output generated by {@link #testDoAfterBodyCurrentToRoot()}. Thus it indirectly tests {@link
     * BreadCrumbTrailTag#doAfterBody()}
     * <p/>
     * <b>Special Scenario</b>: current to root behavior with only mouse over style set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoAfterBodyCurrentToRoot(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals(
            "<span><style>.node-style:hover {__mouseOverStyle__}\n"
                + ".root-node-style:hover {__mouseOverStyle__}\n"
                + ".current-node-style:hover {__mouseOverStyle__}\n"
                + "</style>\n"
                + "node one\n"
                + ":node two\n"
                + ":node three\n", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doEndTag()}.
     * <p/>
     * <b>Special Scenario</b>: default behavior
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoEndTag() throws Exception {
        assertEquals(BodyTagSupport.EVAL_PAGE, breadCrumbTrailTag.doEndTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoEndTag()} has been executed to verify the
     * output generated by {@link #testDoEndTag()}. Thus it indirectly tests {@link BreadCrumbTrailTag#doEndTag()}
     * <p/>
     * <b>Special Scenario</b>: default behavior
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoEndTag(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("", resp.getText());
    }

    /**
     * This method tests the correct behavior of {@link BreadCrumbTrailTag#doEndTag()}.
     * <p/>
     * <b>Special Scenario</b>: styles applied
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoEndTagStyles() throws Exception {
        breadCrumbTrailTag.setMouseOverStyle("__mouseOverStyle__");
        assertEquals(BodyTagSupport.EVAL_PAGE, breadCrumbTrailTag.doEndTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoEndTagStyles()} has been executed to verify the
     * output generated by {@link #testDoEndTagStyles()}. Thus it indirectly tests {@link
     * BreadCrumbTrailTag#doEndTag()}
     * <p/>
     * <b>Special Scenario</b>: styles applied
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoEndTagStyles(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("</span>", resp.getText());
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        breadCrumbTrailTag = new BreadCrumbTrailTag();
        breadCrumbTrailTag.setPageContext(pageContext);
        super.setUp();
    }

    /**
     * This method removes the DynamicPathDiscovery-specific configuration namespaces plus the namespaces of dependant
     * components from the ConfigManager singleton of this class' classloader and then loads the configuration with the
     * given filename/classloader path into the ConfigManager instance.
     *
     * @param filename the config file to be loaded as new DynamicPathDiscovery configuration
     *
     * @throws Exception                in case some unexpected Exception occurs
     * @throws IllegalArgumentException in case the given filename is <tt>null</tt>
     */
    private void loadTagConfiguration(final String filename) throws Exception {
        if (filename == null) {
            throw new IllegalArgumentException("The parameter named [filename] was null");
        }

        // clear config manager data
        removeNamespace(BREAD_CRUMB_TRAIL_TAG_NAMESPACE);
        removeNamespace(BREAD_CRUMB_TRAIL_TAG_OBJECT_FACTORY_NAMESPACE);

        // load the configuration for component,
        ConfigManager.getInstance().add(filename);
    }

    /**
     * This method checks whether the given Object equals the given expected BreadCrumbNode.
     *
     * @param expected the expected node
     * @param o        the object to check for equality with the expected node
     */
    private static void assertEqualNode(final BreadCrumbNode expected, final Object o) {
        if (expected == null) {
            assertNull(o);
        } else {
            assertTrue(o instanceof BreadCrumbNode);
            final BreadCrumbNode node = (BreadCrumbNode) o;
            assertEquals(expected.getUrl(), node.getUrl());
            assertEquals(expected.getTitle(), node.getTitle());
        }
    }

    /**
     * This is a mock {@link javax.servlet.jsp.PageContext} implementation that serves the purpose to have a PageContext
     * implementation which returns <tt>null</tt> in its {@link #getRequest()}  method.
     */
    private class DummyPageContextWrapper extends PageContextWrapper {
        private DummyPageContextWrapper() {
            super(BreadCrumbTrailTagTest.this.pageContext, null);
        }

        public void include(final String string, final boolean b) throws ServletException, IOException {
        }

        public ExpressionEvaluator getExpressionEvaluator() {
            return null;
        }

        public VariableResolver getVariableResolver() {
            return null;
        }

        public ServletRequest getRequest() {
            return null;
        }
    }
}
