/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.meterware.httpunit.WebResponse;
import com.topcoder.web.ui.tag.breadcrumbtrail.BaseBreadCrumbTrailTagTest;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import javax.servlet.jsp.tagext.TagSupport;


/**
 * This class contains all test cases for {@link TemplateNodeFormatterTag}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class TemplateNodeFormatterTagTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";
    /**
     * This is a format string used in test case.
     */
    private static final String FORMAT = "--format--";
    /**
     * This is a template path used in test case.
     */
    private static final String TEMPLATE = "template.jsp";
    /**
     * This is the instance to be tested, created in {@link #setUp()}.
     */
    private TemplateNodeFormatterTag nodeFormatterTag;
    /**
     * This is the current node to be rendered, initialized in {@link #setUp()}.
     */
    private BreadCrumbNode currentNode;

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#TemplateNodeFormatterTag()}.
     */
    public void testNoArgCtor() {
        final TemplateNodeFormatterTag templateNodeFormatterTag = new TemplateNodeFormatterTag();
        assertEquals(null, templateNodeFormatterTag.getTemplate());
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#TemplateNodeFormatterTag(String)}.
     */
    public void testOneArgCtor() {
        TemplateNodeFormatterTag templateNodeFormatterTag = new TemplateNodeFormatterTag(TEMPLATE);
        assertEquals(TEMPLATE, templateNodeFormatterTag.getTemplate());
        // test empty String allowed
        templateNodeFormatterTag = new TemplateNodeFormatterTag(" ");
        assertEquals(" ", templateNodeFormatterTag.getTemplate());
        // test null allowed
        templateNodeFormatterTag = new TemplateNodeFormatterTag(null);
        assertEquals(null, templateNodeFormatterTag.getTemplate());
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#getTemplate()} and {@link
     * TemplateNodeFormatterTag#setTemplate(String)} .
     */
    public void testGetSetTitle() {
        nodeFormatterTag.setTemplate(TEMPLATE);
        assertEquals(TEMPLATE, nodeFormatterTag.getTemplate());
        nodeFormatterTag.setTemplate(null);
        assertEquals(null, nodeFormatterTag.getTemplate());
        nodeFormatterTag.setTemplate(" ");
        assertEquals(" ", nodeFormatterTag.getTemplate());
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>no current node exists
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagNoNode() throws Exception {
        try {
            nodeFormatterTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>current node context attribute has wrong type
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagCurrentNodeWrongType() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, "blah");
        try {
            nodeFormatterTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>current style context attribute has wrong type
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagCurrentStyleWrongType() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new Integer(6));
        try {
            nodeFormatterTag.doStartTag();
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>no current template set
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagNullTemplate() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, currentNode);
        nodeFormatterTag.setTemplate(null);
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagNullTemplate()} has been executed to
     * verify the output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * TemplateNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>no current template set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagNullTemplate(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("", resp.getText().trim());
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>empty current template name set
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagEmptyTemplate() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, currentNode);
        nodeFormatterTag.setTemplate("    ");
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagEmptyTemplate()} has been executed to
     * verify the output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * TemplateNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>empty current template name set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagEmptyTemplate(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("", resp.getText().trim());
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>node exists, but no format is set
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTag() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, currentNode);
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTag()} has been executed to verify the
     * output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * TemplateNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>node exists, but no format is set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTag(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("[__<a href=\"" + URL + "\">" + TITLE + "</a>__]", resp.getText().trim());
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>node exists, but has no title
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagNoTitle() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, new BreadCrumbNode(URL, null));
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagNoTitle()} has been executed to verify
     * the output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * TemplateNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>node exists, but has no title
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagNoTitle(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("[__<a href=\"" + URL + "\">null</a>__]", resp.getText().trim());
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>node and format are set
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagWithFormat() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, currentNode);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, FORMAT);
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagNoTitle()} has been executed to verify
     * the output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * TemplateNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>node and format are set
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagWithFormat(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("[__<a class=\"" + FORMAT + "\" href=\"" + URL + "\">" + TITLE + "</a>__]", resp.getText().trim());
    }

    /**
     * This method tests the correct behavior of {@link TemplateNodeFormatterTag#doStartTag()}.
     * <p/>
     * <b>Special scenario:</b>node and format are set, but format is empty string
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTagWithEmptyFormat() throws Exception {
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, currentNode);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "     ");
        assertEquals(TagSupport.SKIP_BODY, nodeFormatterTag.doStartTag());
    }

    /**
     * This method is called by the cactus framework after {@link #testDoStartTagWithEmptyFormat()} has been executed to
     * verify the output generated by {@link #testDoStartTag()}. Thus it indirectly tests {@link
     * TemplateNodeFormatterTag#doStartTag()}
     * <p/>
     * <b>Special scenario:</b>node and format are set, but format is empty string
     *
     * @param resp the response received from test execution
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void endDoStartTagWithEmptyFormat(final WebResponse resp) throws Exception {
        assertEquals(200, resp.getResponseCode());
        assertEquals("[__<a href=\"" + URL + "\">" + TITLE + "</a>__]", resp.getText().trim());
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        nodeFormatterTag = new TemplateNodeFormatterTag(TEMPLATE);
        nodeFormatterTag.setPageContext(pageContext);
        pageContext.removeAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);
        currentNode = new BreadCrumbNode(URL, TITLE);
    }
}
