/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import junit.framework.TestCase;


/**
 * This class contains all test cases for {@link AbstractNodeMatcher}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class AbstractNodeMatcherTest extends TestCase {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";
    /**
     * This is an empty string for checking empty String argument behavior.
     */
    private static final String EMPTY_STRING = "  ";

    /**
     * This method tests the correct behavior of {@link AbstractNodeMatcher#AbstractNodeMatcher(String,String)}.
     */
    public void testCtor() {
        final DummyNodeMatcher nodeMatcher = new DummyNodeMatcher(TITLE, URL);
        assertEquals(TITLE, nodeMatcher.getTitle());
        assertEquals(URL, nodeMatcher.getUrl());
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractNodeMatcher#AbstractNodeMatcher(String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> url argument
     */
    public void testCtorFailureNullUrl() {
        try {
            new DummyNodeMatcher(TITLE, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractNodeMatcher#AbstractNodeMatcher(String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: empty url argument
     */
    public void testCtorFailureEmptyUrl() {
        try {
            new DummyNodeMatcher(TITLE, EMPTY_STRING);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractNodeMatcher#AbstractNodeMatcher(String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> title argument
     */
    public void testCtorFailureNullTitle() {
        try {
            new DummyNodeMatcher(null, URL);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link AbstractNodeMatcher#AbstractNodeMatcher(String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: empty title argument
     */
    public void testCtorFailureEmptyTitle() {
        try {
            new DummyNodeMatcher(EMPTY_STRING, URL);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link AbstractNodeMatcher#getUrl()}.
     */
    public void testGetUrl() {
        final DummyNodeMatcher node = new DummyNodeMatcher(TITLE, URL);
        assertEquals(URL, node.getUrl());
    }

    /**
     * This method tests the correct behavior of {@link AbstractNodeMatcher#getTitle()}.
     */
    public void testGetTitle() {
        final DummyNodeMatcher node = new DummyNodeMatcher(TITLE, URL);
        assertEquals(TITLE, node.getTitle());
    }

    /**
     * This is a mock class for testing the functionality in {@link AbstractNodeMatcher}. The sole function of this
     * class is to have an instantiatable AbstractNodeMatcher subclass.
     */
    private static class DummyNodeMatcher extends AbstractNodeMatcher {
        /**
         * This constructor simply calls the super constructor.
         *
         * @param title A non-<tt>null</tt>, non-empty (trim'd) title to use
         * @param url   A non-<tt>null</tt>, non-empty (trim'd) url to use
         *
         * @throws IllegalArgumentException if either arg is <tt>null</tt> or an empty (trim'd) string
         */
        private DummyNodeMatcher(final String title, final String url) {
            super(title, url);
        }

        /**
         * This method determines whether the specified node matches this node matcher.
         *
         * @param node A non-<tt>null</tt> node to test
         *
         * @return <tt>true</tt> if the node matches, <tt>false</tt> otherwise
         *
         * @throws IllegalArgumentException if the node is not <tt>null</tt>
         */
        public boolean matches(final BreadCrumbNode node) {
            return false;
        }
    }
}
