/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.web.ui.tag.breadcrumbtrail.BaseBreadCrumbTrailTagTest;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;


/**
 * This class contains all test cases for {@link DynamicPathDiscovery}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class DynamicPathDiscoveryTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher1;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher2;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher4;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher5;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher6;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher7;
    /**
     * This is a NodeMatcher used in test cases, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher8;
    /**
     * This is the instance to be tested, initialized in {@link #setUp()}.
     */
    private DynamicPathDiscovery dynamicPathDiscovery;

    /**
     * This method tests the correct behavior of {@link DynamicPathDiscovery#DynamicPathDiscovery()}.
     */
    public void testOneArgCtor() {
        new DynamicPathDiscovery();
    }

    /**
     * This method tests the correct behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)}.
     * <p/>
     * <b>Special scenario:</b> path is not found
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPath() throws Exception {
        BreadCrumbNode[] path = dynamicPathDiscovery.getPath(new BreadCrumbNode("blah", null), pageContext);
        assertEquals(null, path);
    }

    /**
     * This method tests the correct behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)}.
     * <p/>
     * <b>Special scenario:</b> path is a root node
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathRoot() throws Exception {
        BreadCrumbNode[] path = dynamicPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher1.getUrl(), null),
            pageContext);
        assertEquals(1, path.length);
        assertEquals(nodeMatcher1.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher1.getUrl(), path[0].getUrl());

    }

    /**
     * This method tests the correct behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)}.
     * <p/>
     * <b>Special scenario:</b> path is created dynamically
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathDynamic() throws Exception {
        // setup a currentPath by discovering one before
        testGetPathRoot();

        //now make the dynamic discovery
        BreadCrumbNode[] path = dynamicPathDiscovery.getPath(new BreadCrumbNode("blah", null), pageContext);
        assertEquals(2, path.length);
        assertEquals(nodeMatcher1.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher1.getUrl(), path[0].getUrl());
        assertEquals(null, path[1].getTitle());
        assertEquals("blah", path[1].getUrl());

    }

    /**
     * This method tests the correct behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)}.
     * <p/>
     * <b>Special scenario:</b> path is found normally
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathNormal() throws Exception {
        BreadCrumbNode[] path = dynamicPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher2.getUrl(), null),
            pageContext);
        assertEquals(2, path.length);
        assertEquals(nodeMatcher1.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher1.getUrl(), path[0].getUrl());
        assertEquals(nodeMatcher2.getTitle(), path[1].getTitle());
        assertEquals(nodeMatcher2.getUrl(), path[1].getUrl());

    }

    /**
     * This method tests the correct behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)}.
     * <p/>
     * <b>Special scenario:</b> two paths exist, the shorter one is chosen
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathShortest() throws Exception {
        BreadCrumbNode[] path = dynamicPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher4.getUrl(), null),
            pageContext);
        assertEquals(2, path.length);
        assertEquals(nodeMatcher5.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher5.getUrl(), path[0].getUrl());
        assertEquals(nodeMatcher4.getTitle(), path[1].getTitle());
        assertEquals(nodeMatcher4.getUrl(), path[1].getUrl());

    }

    /**
     * This method tests the correct behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)}.
     * <p/>
     * <b>Special scenario:</b> cycle in sitemap
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathCycle() throws Exception {
        BreadCrumbNode[] path = dynamicPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher7.getUrl(), null),
            pageContext);
        assertEquals(3, path.length);
        assertEquals(nodeMatcher5.getTitle(), path[0].getTitle());
        assertEquals(nodeMatcher5.getUrl(), path[0].getUrl());
        assertEquals(nodeMatcher6.getTitle(), path[1].getTitle());
        assertEquals(nodeMatcher6.getUrl(), path[1].getUrl());
        assertEquals(nodeMatcher7.getTitle(), path[2].getTitle());
        assertEquals(nodeMatcher7.getUrl(), path[2].getUrl());

    }

    /**
     * This method tests the correct behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)}.
     * <p/>
     * <b>Special scenario:</b> two nodes build a cycle and have no root
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathNoRoot() throws Exception {
        pageContext.removeAttribute(DynamicPathDiscovery.CURRENT_PATH);
        BreadCrumbNode[] path = dynamicPathDiscovery.getPath(new BreadCrumbNode(nodeMatcher8.getUrl(), null),
            pageContext);
        assertEquals(null, path);
    }

    /**
     * This method tests the correct failure behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>currentNode arg is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureNullNode() throws Exception {
        try {
            dynamicPathDiscovery.getPath(null, pageContext);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>pageContext arg is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureNullPageContext() throws Exception {
        try {
            dynamicPathDiscovery.getPath(new BreadCrumbNode("aa", null), null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory cannot be loaded
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureEmptyObjectFactoryConfiguration() throws Exception {
        loadDynamicPathDiscoveryConfiguration("dynamicPathDiscovery_empty.xml");
        try {
            dynamicPathDiscovery.getPath(new BreadCrumbNode("aa", null), pageContext);
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory cannot be loaded
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureMalformedObjectFactoryConfiguration() throws Exception {
        loadDynamicPathDiscoveryConfiguration("dynamicPathDiscovery_malformed.xml");
        try {
            dynamicPathDiscovery.getPath(new BreadCrumbNode("aa", null), pageContext);
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory cannot construct SiteDiscovery
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureUnconstructableObjectFactoryConfiguration() throws Exception {
        loadDynamicPathDiscoveryConfiguration("dynamicPathDiscovery_unconstructable.xml");
        try {
            dynamicPathDiscovery.getPath(new BreadCrumbNode("aa", null), pageContext);
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory constructs siteDiscovery with invalid data
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureInvalidSitemapConfiguration() throws Exception {
        loadDynamicPathDiscoveryConfiguration("dynamicPathDiscovery_invalidSitemap.xml");
        try {
            dynamicPathDiscovery.getPath(new BreadCrumbNode("aa", null), pageContext);
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link DynamicPathDiscovery#getPath(BreadCrumbNode,
     *javax.servlet.jsp.PageContext)} .
     * <p/>
     * <b>Failure scenario:</b>ObjectFactory constructs siteDiscovery with malformed data
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetPathFailureMalformedSitemapConfiguration() throws Exception {
        loadDynamicPathDiscoveryConfiguration("dynamicPathDiscovery_malformedSitemap.xml");
        try {
            dynamicPathDiscovery.getPath(new BreadCrumbNode("aa", null), pageContext);
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        nodeMatcher1 = new RegexNodeMatcher("aa1", "uu1", "uu1");
        nodeMatcher2 = new RegexNodeMatcher("aa2", "uu2", "uu2");
        nodeMatcher4 = new RegexNodeMatcher("aa4", "uu4", "uu4");
        nodeMatcher5 = new RegexNodeMatcher("aa5", "uu5", "uu5");
        nodeMatcher6 = new RegexNodeMatcher("a", "uu6", "uu6");
        nodeMatcher7 = new RegexNodeMatcher("aa7", "uu7", "uu7");
        nodeMatcher8 = new RegexNodeMatcher("aa8", "uu8", "uu8");
        pageContext.removeAttribute(DynamicPathDiscovery.CURRENT_SITEMAP);
        pageContext.removeAttribute(DynamicPathDiscovery.CURRENT_REVERSE_SITEMAP);
        pageContext.removeAttribute(DynamicPathDiscovery.CURRENT_PATH);
        dynamicPathDiscovery = new DynamicPathDiscovery();
    }

    /**
     * This method removes the DynamicPathDiscovery-specific configuration namespaces plus the namespaces of dependant
     * components from the ConfigManager singleton of this class' classloader and then loads the configuration with the
     * given filename/classloader path into the ConfigManager instance.
     *
     * @param filename the config file to be loaded as new DynamicPathDiscovery configuration
     *
     * @throws Exception                in case some unexpected Exception occurs
     * @throws IllegalArgumentException in case the given filename is <tt>null</tt>
     */
    private void loadDynamicPathDiscoveryConfiguration(final String filename) throws Exception {
        if (filename == null) {
            throw new IllegalArgumentException("The parameter named [filename] was null");
        }

        // clear config manager data
        removeNamespace(DYNAMIC_PATH_DISCOVERY_OBJECT_FACTORY_NAMESPACE);

        // load the configuration for component,
        ConfigManager.getInstance().add(filename);
    }
}
