/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;
import junit.framework.TestCase;


/**
 * This class contains all test cases for {@link NodePair}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class NodePairTest extends TestCase {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";
    /**
     * This is a BreadCrumbNode instance used in test cases, initialized in {@link #setUp()}.
     */
    private BreadCrumbNode breadCrumbNode;
    /**
     * This is a NodeMatcher instance used in test cases, initialized in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher;

    /**
     * This method tests the correct behavior of {@link NodePair#NodePair(BreadCrumbNode,NodeMatcher)}.
     */
    public void testCtor() {
        NodePair nodePair = new NodePair(breadCrumbNode, nodeMatcher);
        assertEquals(nodeMatcher, nodePair.getNodeMatcher());
        assertEquals(breadCrumbNode, nodePair.getNode());

        // check with null matcher
        nodePair = new NodePair(breadCrumbNode, null);
        assertEquals(null, nodePair.getNodeMatcher());
        assertEquals(breadCrumbNode, nodePair.getNode());

        final BreadCrumbNode crumbNode = new BreadCrumbNode(URL, null);
        // check with null title in node to be replaced with title from matcher
        nodePair = new NodePair(crumbNode, nodeMatcher);
        assertEquals(nodeMatcher, nodePair.getNodeMatcher());
        assertNotSame(crumbNode, nodePair.getNode());
        assertEquals(TITLE, nodePair.getNode().getTitle());

    }

    /**
     * This method tests the correct failure behavior of {@link NodePair#NodePair(BreadCrumbNode,NodeMatcher)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> node argument
     */
    public void testCtorFailureNullNode() {
        try {
            new NodePair(null, nodeMatcher);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodePair#getNode()}.
     */
    public void testGetNode() {
        assertEquals(breadCrumbNode, new NodePair(breadCrumbNode, nodeMatcher).getNode());
    }

    /**
     * This method tests the correct behavior of {@link NodePair#getNodeMatcher()}.
     */
    public void testGetTitle() {
        NodePair nodePair = new NodePair(breadCrumbNode, nodeMatcher);
        assertEquals(nodeMatcher, nodePair.getNodeMatcher());
        // check with null matcher
        nodePair = new NodePair(breadCrumbNode, null);
        assertEquals(null, nodePair.getNodeMatcher());
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        breadCrumbNode = new BreadCrumbNode(URL, TITLE);
        nodeMatcher = new RegexNodeMatcher(TITLE, URL, URL);
    }
}
