/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import junit.framework.TestCase;


/**
 * This class contains all test cases for {@link RegexNodeMatcher}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class RegexNodeMatcherTest extends TestCase {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";
    /**
     * This is a value used as valid pattern argument in testcases.
     */
    private static final String PATTERN = "u://rl.*";
    /**
     * This is an empty string for checking empty String argument behavior.
     */
    private static final String EMPTY_STRING = "  ";
    /**
     * This is the instance to be tested, created in {@link #setUp()}.
     */
    private RegexNodeMatcher nodeMatcher;

    /**
     * This method tests the correct behavior of {@link RegexNodeMatcher#RegexNodeMatcher(String,String,String)}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtor() throws Exception {
        final RegexNodeMatcher regexNodeMatcher = new RegexNodeMatcher(TITLE, URL, PATTERN);
        assertEquals(TITLE, regexNodeMatcher.getTitle());
        assertEquals(URL, regexNodeMatcher.getUrl());
        assertEquals(PATTERN, regexNodeMatcher.getNodePattern());
    }

    /**
     * This method tests the correct failure behavior of {@link RegexNodeMatcher#RegexNodeMatcher(String,String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> url argument
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtorFailureNullUrl() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, null, PATTERN);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link RegexNodeMatcher#RegexNodeMatcher(String,String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: empty url argument
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtorFailureEmptyUrl() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, EMPTY_STRING, PATTERN);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link RegexNodeMatcher#RegexNodeMatcher(String,String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> title argument
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtorFailureNullTitle() throws Exception {
        try {
            new RegexNodeMatcher(null, URL, PATTERN);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link RegexNodeMatcher#RegexNodeMatcher(String,String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: empty title argument
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtorFailureEmptyTitle() throws Exception {
        try {
            new RegexNodeMatcher(EMPTY_STRING, URL, PATTERN);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link RegexNodeMatcher#RegexNodeMatcher(String,String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: <tt>null</tt> pattern argument
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtorFailureNullPattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, null);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link RegexNodeMatcher#RegexNodeMatcher(String,String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: empty pattern argument
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtorFailureEmptyPattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, EMPTY_STRING);
            fail("should throw");
        } catch (IllegalArgumentException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct failure behavior of {@link RegexNodeMatcher#RegexNodeMatcher(String,String,
     *String)}.
     * <p/>
     * <b>Failure reason</b>: invalid pattern argument
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testCtorFailureInvalidPattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, "a(");
            fail("should throw");
        } catch (BreadCrumbException expected) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link RegexNodeMatcher#getUrl()}.
     */
    public void testGetUrl() {
        assertEquals(URL, nodeMatcher.getUrl());
    }

    /**
     * This method tests the correct behavior of {@link RegexNodeMatcher#getTitle()}.
     */
    public void testGetTitle() {
        assertEquals(TITLE, nodeMatcher.getTitle());
    }

    /**
     * This method tests the correct behavior of {@link RegexNodeMatcher#getNodePattern()}.
     */
    public void testGetPattern() {
        assertEquals(PATTERN, nodeMatcher.getNodePattern());
    }

    /**
     * This method tests the correct behavior of {@link RegexNodeMatcher#matches(BreadCrumbNode)}.
     */
    public void testMatches() {
        assertTrue(nodeMatcher.matches(new BreadCrumbNode(URL, TITLE)));
        assertTrue(nodeMatcher.matches(new BreadCrumbNode(URL, null)));
        assertTrue(nodeMatcher.matches(new BreadCrumbNode(URL, EMPTY_STRING)));
        assertFalse(nodeMatcher.matches(new BreadCrumbNode("blah", TITLE)));
        assertFalse(nodeMatcher.matches(new BreadCrumbNode("()8u989", TITLE)));
    }

    /**
     * This method tests {@link RegexNodeMatcher#equals(Object)} for correctness.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testEquals() throws Exception {
        //null
        assertFalse(nodeMatcher.equals(null));
        //reflexive
        assertTrue(nodeMatcher.equals(nodeMatcher));
        // wrong type
        assertFalse(nodeMatcher.equals(new BreadCrumbNode(URL, TITLE)));
        //subclass must not match
        assertFalse(nodeMatcher.equals(new RegexNodeMatcher(TITLE, URL, PATTERN) {
        }));

        assertTrue(nodeMatcher.equals(new RegexNodeMatcher(TITLE, URL, PATTERN)));

        // now check all fields are included in equals calculation
        //title
        assertFalse(nodeMatcher.equals(new RegexNodeMatcher("blah", URL, PATTERN)));
        //url
        assertFalse(nodeMatcher.equals(new RegexNodeMatcher(TITLE, "blah", PATTERN)));
        //pattern
        assertFalse(nodeMatcher.equals(new RegexNodeMatcher(TITLE, URL, "blah")));
    }

    /**
     * This method tests {@link RegexNodeMatcher#hashCode()} for correctness.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testHashCode() throws Exception {
        // is equal for equal objects
        assertEquals(nodeMatcher.hashCode(), new RegexNodeMatcher(TITLE, URL, PATTERN).hashCode());
        // title is in hashcode
        assertFalse(nodeMatcher.hashCode() == new RegexNodeMatcher("blah", URL, PATTERN).hashCode());
        // url is in hashCode
        assertFalse(nodeMatcher.hashCode() == new RegexNodeMatcher(TITLE, "blah", PATTERN).hashCode());
        // pattern is in hashCode
        assertFalse(nodeMatcher.hashCode() == new RegexNodeMatcher(TITLE, URL, "blah").hashCode());
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        nodeMatcher = new RegexNodeMatcher(TITLE, URL, PATTERN);
    }
}
